using System;
using System.Drawing;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace Mizel.Neuro
{

    public enum QStyle { q3x3 = 9, q4x4 = 16, q5x5 = 25, q6x6 = 36, q7x7 = 49, q8x8 = 64 }

    [Serializable]
    public class DatenMuster:IComponent,ICloneable
    {
        int unit = 0;
        int unitlength = 0;
        int wholewidth = 0;
        Bitmap _bmp;

        public DatenMuster()
        {
            INIT(16, 300);
        }

        public DatenMuster(int steine, Size size)
        {
            INIT(steine, size.Width);
        }
        public DatenMuster(int steine, int width)
        {
            INIT(steine, width);
        }

        void INIT(int steine, int width)
        {
            wholewidth = width;
            _qstyle = (QStyle)steine;
            unit = (int)Math.Sqrt(steine);
            unitlength = wholewidth / unit;
            _bmp = new Bitmap(unit, unit);
        }

        public DatenMuster(DatenMuster dm)
        {
            _name = dm._name;
            _outputD = dm._outputD;
            _qstyle = dm._qstyle;
            _bmp = (Bitmap)dm.Bitmap.Clone();

        }

        public Bitmap Bitmap
        {
            get { return _bmp; }
            set { _bmp = value; }
        }


        public int Units
        {
            get { return unit; }
        }
        public int UnitsLength
        {
            get { return unitlength; }
        }

        public int UnitsLengthsdf
        {
            get { return unit; }
        }

        private QStyle _qstyle = QStyle.q3x3;
        public QStyle StyleSize
        {
            get { return _qstyle; }
            set { _qstyle = value;
            ChangeQStyle((int)_qstyle);
            }
        }


            

        public string AS_TEXT
        {
            get { return AsText(); }            
        }

        string AsText()
        {
            string ret = "";
            
            for (int y = 0; y < Bitmap.Height; y++)
            {
                for (int x = 0; x < Bitmap.Width; x++)
                {
                    ret+=PixelGet(x,y)?"1 ":"0 ";
                }
                ret += "\r\n";
            }
            return ret;
        }


        public int this[int index]
        {
            get
            {
                if (this.unit == 0)
                    this.unit = this.Bitmap.Height;
                  int x=index%this.unit;
                  int y=index/this.unit;
                  return PixelGet(x, y) ? 1 : 0;
            }
            set {
                int x = index % this.unitlength;
                int y = index / this.unitlength;
                PixelChange(x, y);
            }
        }
	



        public void PixelINIT()
        {
            for (int x = 0; x < _bmp.Width; x++)
                for (int y = 0; y < _bmp.Height; y++)
            {
                this.PixelSet(x,y,false);
            }
        }
        public void PixelInvert()
        {
            for (int x = 0; x < _bmp.Width; x++)
                for (int y = 0; y < _bmp.Height; y++)
                {
                    this.PixelChange(x, y);
                }
        }
        public void PixelChange(int x, int y)
        {
            bool ncol=!PixelGet(x,y);
            PixelSet(x, y,ncol);
        }
        

        public bool PixelGet(int x, int y)
        {
            if (x >= _bmp.Width || y >= _bmp.Height)
                return false;

                Color c=_bmp.GetPixel(x, y);
            int t=c.R+c.G+c.B;
            return  t<=0;
        }

        public void PixelSet(int x, int y,bool value)
        {
            if (x * y >= _bmp.Width * _bmp.Height)
                return;

            int col=value?0:255;
            _bmp.SetPixel(x, y,Color.FromArgb(col,col,col));
        }

        private int _outputD = 0;
        public int OutputD
        {
            get { return _outputD; }
            set { _outputD = value; }
        }

        public string OutputDSymbol
        {
            get { return new String(new char[]{(char)_outputD}); }
            set
            {
                if (value.Length>0)
                _outputD = (int)value[0];
            }
        }
	



        #region IComponent Members
        public event EventHandler Disposed;
        ISite site;
        [EditorBrowsable(EditorBrowsableState.Never),Browsable(false)]
        public ISite Site
        {
            get
            {
                return site;
            }
            set
            {
                site = value;
            }
        }

        public void Dispose()
        {
            if (Disposed!=null)
                Disposed(this, EventArgs.Empty);
        }

        #endregion

        internal void ReadBitmap(Bitmap bitmap)
        {
            Graphics g = Graphics.FromImage(this.Bitmap);
            g.DrawImage(bitmap, new Point(0, 0));
        }

        public void ChangeQStyle(string qstylename)
        {
            QStyle q = (QStyle)Enum.Parse(typeof(QStyle), qstylename, true);
            ChangeQStyle(q);
        }
        public void ChangeQStyle(int steine)
        {
            QStyle q = (QStyle)steine;
            ChangeQStyle(q);
        }

        public void ChangeQStyle(QStyle q)
        {
            Bitmap tmpbmp = (Bitmap)this.Bitmap.Clone();
            INIT((int)q, wholewidth);
            ReadBitmap(tmpbmp);
        }

        private string _name;
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }


        #region ICloneable Members

        public object Clone()
        {
            return new DatenMuster(this);
        }

        #endregion
    }
}
