
using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Collections;

namespace Mizel.Neuro
{
    [Serializable]
    public class DatenNetz
    {
        double[] _input_arr;
        double[] _hidden_arr;
        double[] _output_arr;

        double[,] _zwischen1_2;
        double[,] _zwischen2_3;

        double[] _bias_h;
        double[] _bias_o;

        public DatenNetz()
        {
            INIT_VARS(16, 16, 8);
        }

        public DatenNetz(int input_size, int hidden_size, int output_size)
        {
            INIT_VARS(input_size, hidden_size, output_size);
        }

        void INIT_VARS(int input_size, int hidden_size, int output_size)
        {
            this._input_arr = new double[input_size];
            this._hidden_arr = new double[hidden_size];
            this._output_arr = new double[output_size];


            SizeChanged(this, null);
        }


        void SizeChanged(object sender, object who)
        {
            _zwischen1_2 = new double[_input_arr.Length, _hidden_arr.Length];
            _zwischen2_3 = new double[_hidden_arr.Length, _output_arr.Length];

            _bias_h = new double[_hidden_arr.Length];
            _bias_o = new double[_output_arr.Length];

            INIT_Zwischen();
        }

        public void INIT_Zwischen()
        {
            ResetZwischen();
            ResetBias();
        }






        public double[,] Gewichte1_2
        {
            get { return _zwischen1_2; }
            set { _zwischen1_2 = value; }
        }

        public double[,] Gewichte2_3
        {
            get { return _zwischen2_3; }
            set { _zwischen2_3 = value; }
        }

        public double[] BIAS_H
        {
            get { return _bias_h; }
            set { _bias_h = value; }
        }
        public double[] BIAS_O
        {
            get { return _bias_o; }
            set { _bias_o = value; }
        }

	

        [Category("Size")]
        public int InputSize
        {
            get { return _input_arr.Length; }
            set { _input_arr =new double[value];
            SizeChanged(this, _input_arr);
            }
        }
        [Category("Size")]
        public int HiddenSize
        {
            get { return _hidden_arr.Length; }
            set
            {
                _hidden_arr = new double[value];
                SizeChanged(this, _hidden_arr);
            }
        }
        [Category("Size")]
        public int OutputSize
        {
            get { return _output_arr.Length; }
            set
            {
                _output_arr = new double[value];
                SizeChanged(this, _output_arr);
            }
        }


        [Category("Value")]
        public double[] Input
        {
            get { return _input_arr; }
            set { _input_arr = value; }
        }
        [Category("Value")]
        public double[] Hidden
        {
            get { return _hidden_arr; }
            set { _hidden_arr = value; }
        }
        [Category("Value")]
        public double[] Output
        {
            get { return _output_arr; }
            set { _output_arr = value; }
        }



        private DatenMusterCollection _dms=new DatenMusterCollection();
        [Category("Muster")]        
        public DatenMusterCollection MusterItems
        {
            get { return _dms; }
            set { if(value!=null)_dms = value; }
        }


        internal void ResetBias()
        {
            _bias_h = new double[_hidden_arr.Length];
            _bias_o = new double[_output_arr.Length];
            
            //BIAS J und K
            for (int j = 0; j < _bias_h.Length; j++)
                _bias_h[j] = 0.1;

            for (int k = 0; k < _bias_o.Length; k++)
                _bias_o[k] = 0.1;
        }
        internal void ResetZwischen()
        {
            Random rand = new Random();
            //INPUT_HIDDEN
            for (int ha = 0; ha < _input_arr.Length; ha++)
                for (int oa = 0; oa < _hidden_arr.Length; oa++)
                {

                    double zu = rand.NextDouble() + 0.1;
                    double ist = rand.NextDouble() + 0.1;
                    bool negation = rand.NextDouble() > 0.5;
                    if (negation)
                        ist = ist * -1;
                    _zwischen1_2[ha, oa] = ist;
                }

            //HIDDEN_OUTPUT
            for (int ha = 0; ha < _hidden_arr.Length; ha++)
                for (int oa = 0; oa < _output_arr.Length; oa++)
                {

                    double zu = rand.NextDouble() + 0.1;
                    double ist = rand.NextDouble() + 0.1;
                    bool negation = rand.NextDouble() > 0.5;
                    if (negation)
                        ist = ist * -1;
                    _zwischen2_3[ha, oa] = ist;
                }
        }
    }
}
